import 'package:basic_ui/src/providers/theme_provider.dart';
import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:flutter_color/flutter_color.dart';

import '../../basic_ui.dart';

/// Creates a BasicApp.
class BasicApp extends HookWidget {
  const BasicApp({
    Key? key,
    this.title = '',
    required this.home,
    this.theme,
  }) : super(key: key);

  /// A one-line description used by the device to identify the app for the user.
  ///
  /// On Android the titles appear above the task manager's app snapshots
  /// which are displayed when the user presses the "recent apps" button.
  /// On iOS this value cannot be used. CFBundleDisplayName from the app's
  /// Info.plist is referred to instead whenever present, CFBundleName otherwise.
  /// On the web it is used as the page title, which shows up in
  /// the browser's list of open tabs.
  final String title;

  /// The widget for the home page of the app.
  final Widget home;

  /// Default visual properties, like colors fonts and colors.
  final BasicTheme? theme;

  @override
  Widget build(BuildContext context) {
    // Apply the theme to the application
    final themeProviderValue = useProvider(themeProvider);
    final theme = themeProviderValue.theme;

    // Generate dark and light color from primaryColor
    final primaryDarkColor = theme.primaryColor.darker(10);
    final primaryLightColor = theme.primaryColor.lighter(25);

    useEffect(() {
      Future.delayed(Duration.zero, () {
        themeProviderValue.changeTheme(this.theme);
      });
    }, [this.theme]);

    return MaterialApp(
      title: title,
      theme: ThemeData(
        accentColor: theme.primaryColor,
        primaryColor: theme.primaryColor,
        primaryColorDark: primaryDarkColor,
        primaryColorLight: primaryLightColor,
        visualDensity: VisualDensity.adaptivePlatformDensity,
        textSelectionTheme: TextSelectionThemeData(
          selectionColor: primaryLightColor,
          selectionHandleColor: primaryDarkColor,
          cursorColor: theme.primaryColor,
        ),
        buttonColor: theme.primaryColor,
        fontFamily: theme.fontFamily,
        textButtonTheme: TextButtonThemeData(
          style: ButtonStyle(
            shape: MaterialStateProperty.all<RoundedRectangleBorder>(
              const RoundedRectangleBorder(),
            ),
            overlayColor: MaterialStateColor.resolveWith(
              (states) => theme.borderColor,
            ),
          ),
        ),
        elevatedButtonTheme: ElevatedButtonThemeData(
          style: ButtonStyle(
            shape: MaterialStateProperty.all<RoundedRectangleBorder>(
              const RoundedRectangleBorder(),
            ),
            backgroundColor: MaterialStateColor.resolveWith(
              (states) => theme.primaryColor,
            ),
            overlayColor: MaterialStateColor.resolveWith(
              (states) => primaryDarkColor,
            ),
            elevation: MaterialStateProperty.resolveWith((states) => 0),
          ),
        ),
        outlinedButtonTheme: OutlinedButtonThemeData(
          style: ButtonStyle(
            shape: MaterialStateProperty.all<RoundedRectangleBorder>(
              RoundedRectangleBorder(side: BorderSide(color: theme.fontColor)),
            ),
            overlayColor: MaterialStateColor.resolveWith(
              (states) => theme.borderColor,
            ),
          ),
        ),
      ),
      home: home,
    );
  }
}
