import 'package:basic_ui/basic_ui.dart';
import 'package:basic_ui/src/providers/theme_provider.dart';
import 'package:basic_ui/src/widgets/basic_color.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

import '../utils/get_luminance.dart';
import 'basic_text.dart';

/// Creates a [BasicAppBar].
class BasicAppBar extends HookWidget implements PreferredSizeWidget {
  const BasicAppBar({
    Key? key,
    required this.title,
    this.actions,
    this.enableSearchBar = false,
    this.searchBarPlaceHolder,
    this.onClear,
    this.onSubmitted,
    this.tabs,
    this.appBarTheme,
  }) : super(key: key);

  /// Display title to the app bar.
  final String title;

  /// A list of Widgets to display in a row after the [title].
  final List<Widget>? actions;

  final BasicAppBarTheme? appBarTheme;

  /// Set this to true to make app bar title display search bar.
  final bool enableSearchBar;

  /// Search bar placeholder
  final String? searchBarPlaceHolder;

  /// The callback that is called when the clear button is tapped.
  final Function? onClear;

  /// The callback when search query submitted.
  final Function(String)? onSubmitted;

  /// A list of Widgets to display in [TabBar]
  final List<Widget>? tabs;

  List<Widget> _createActionMenu(TextEditingController searchBarController) {
    return [
      IconButton(
        icon: const Icon(Icons.close),
        onPressed: () {
          searchBarController.clear();
          if (onClear != null) onClear!();
        },
      )
    ];
  }

  @override
  Widget build(BuildContext context) {
    final theme = useProvider(themeProvider).theme;

    final searchBarController = TextEditingController();
    final backgroundColor = appBarTheme?.color ?? theme.appBar.color;
    Color textColor = BasicColor.fontColor;
    final double luminance = backgroundColor.getLuminance();

    final usedElevation = appBarTheme?.elevation ?? theme.appBar.elevation;
    final usedBorderWidth =
        appBarTheme?.borderWidth ?? theme.appBar.borderWidth;
    final usedBorderColor =
        appBarTheme?.borderColor ?? theme.appBar.borderColor;

    luminance > 0.5
        ? textColor = BasicColor.fontColor
        : textColor = Colors.white;

    TabBar? tabBar;
    if (tabs != null) tabBar = TabBar(tabs: tabs!);

    return AppBar(
      title: !enableSearchBar
          ? BasicText(
              title,
              color: textColor,
              fontWeight: FontWeight.w600,
              fontSize: 17,
            )
          : TextField(
              autofocus: true,
              controller: searchBarController,
              decoration: InputDecoration(
                border: InputBorder.none,
                focusedBorder: InputBorder.none,
                enabledBorder: InputBorder.none,
                errorBorder: InputBorder.none,
                disabledBorder: InputBorder.none,
                hintText: searchBarPlaceHolder,
                hintStyle: TextStyle(color: textColor.withAlpha(200)),
              ),
              onSubmitted: onSubmitted,
              textInputAction: TextInputAction.search,
              style: TextStyle(color: textColor),
            ),
      actions:
          !enableSearchBar ? actions : _createActionMenu(searchBarController),
      elevation: usedElevation,
      brightness: luminance > 0.5 ? Brightness.light : Brightness.dark,
      backgroundColor: backgroundColor,
      iconTheme: IconThemeData(color: textColor),
      bottom: tabBar ??
          PreferredSize(
            preferredSize: Size.fromHeight(usedBorderWidth),
            child: Container(
              color: usedBorderColor,
              height: usedBorderWidth,
            ),
          ),
    );
  }

  @override
  Size get preferredSize => Size.fromHeight(
      tabs == null ? kToolbarHeight : kToolbarHeight + kTextTabBarHeight);
}
