import 'package:basic_ui/basic_ui.dart';
import 'package:basic_ui/src/models/basic_theme.dart';
import 'package:basic_ui/src/providers/theme_provider.dart';
import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

import '../utils/get_luminance.dart';
import 'basic_color.dart';
import 'basic_text.dart';

enum BasicButtonVariant { primary, outlined, text }

/// Buttons give people a way to trigger an action.
/// They’re typically found in forms, dialog panels, and dialogs.
/// Some buttons are specialized for particular tasks,
/// such as navigation, repeated actions, or presenting menus.
class BasicButton extends HookWidget {
  const BasicButton(
    this.text, {
    Key? key,
    required this.onPressed,
    this.variant = BasicButtonVariant.primary,
  }) : super(key: key);

  /// Text to display in button.
  final String text;

  /// The callback when button is pressed.
  final void Function()? onPressed;

  /// Button variant.
  final BasicButtonVariant variant;

  @override
  Widget build(BuildContext context) {
    final theme = useProvider(themeProvider).theme;

    switch (variant) {
      case BasicButtonVariant.primary:
        return _PrimaryButton(onPressed: onPressed, text: text, theme: theme);
      case BasicButtonVariant.outlined:
        return _OutlinedButton(onPressed: onPressed, text: text, theme: theme);
      case BasicButtonVariant.text:
        return _TextButton(onPressed: onPressed, theme: theme, text: text);

      default:
        return _PrimaryButton(onPressed: onPressed, text: text, theme: theme);
    }
  }
}

class _TextButton extends StatelessWidget {
  const _TextButton({
    Key? key,
    this.onPressed,
    required this.text,
    required this.theme,
  }) : super(key: key);

  final void Function()? onPressed;
  final String text;
  final BasicTheme theme;

  @override
  Widget build(BuildContext context) {
    return BasicTap(
      onTap: onPressed,
      child: Container(
        padding: const EdgeInsets.symmetric(vertical: 14, horizontal: 8),
        child: BasicText(
          text,
          color: theme.primaryColor,
          variant: BasicTextVariant.label,
        ),
      ),
    );
  }
}

class _OutlinedButton extends StatelessWidget {
  const _OutlinedButton({
    Key? key,
    this.onPressed,
    required this.text,
    required this.theme,
  }) : super(key: key);

  final void Function()? onPressed;
  final String text;
  final BasicTheme theme;

  @override
  Widget build(BuildContext context) {
    return BasicTap(
      onTap: onPressed,
      child: Padding(
        padding: const EdgeInsets.symmetric(vertical: 6.0),
        child: BasicCard(
          padding: const EdgeInsets.symmetric(vertical: 8, horizontal: 14),
          borderWidth: 1,
          child: BasicText(
            text,
            color: theme.fontColor,
            variant: BasicTextVariant.label,
          ),
        ),
      ),
    );
  }
}

class _PrimaryButton extends StatelessWidget {
  const _PrimaryButton({
    Key? key,
    this.onPressed,
    required this.text,
    required this.theme,
  }) : super(key: key);

  final void Function()? onPressed;
  final String text;
  final BasicTheme theme;

  @override
  Widget build(BuildContext context) {
    return BasicTap(
      onTap: onPressed,
      child: Padding(
        padding: const EdgeInsets.symmetric(vertical: 6.0),
        child: Container(
          padding: const EdgeInsets.symmetric(vertical: 8, horizontal: 14),
          decoration: BoxDecoration(
            color: theme.primaryColor,
          ),
          child: BasicText(
            text,
            color: theme.primaryColor.getLuminance() > 0.5
                ? BasicColor.fontColor
                : Colors.white,
            variant: BasicTextVariant.label,
          ),
        ),
      ),
    );
  }
}
