import 'package:basic_ui/src/providers/theme_provider.dart';
import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

/// Cards contain content and actions about a single subject.
class BasicCard extends HookWidget {
  const BasicCard({
    Key? key,
    this.color,
    this.child,
    this.padding,
    this.margin,
    this.onTap,
    this.borderRadius,
    this.borderWidth,
    this.elevation,
  }) : super(key: key);

  /// The widget below this widget in the tree.
  final Widget? child;

  /// [BasicCard] color.
  final Color? color;

  /// The empty space that surrounds the card.
  final EdgeInsetsGeometry? padding;

  /// The empty space that surrounds the card.
  final EdgeInsetsGeometry? margin;

  /// The callback when card is tapped
  final Function()? onTap;

  /// Property to set card border radius, if null the default value is used.
  final double? borderRadius;

  /// Property to set card border width, if null the default value is used.
  final double? borderWidth;

  /// Card shadow size.
  final double? elevation;

  @override
  Widget build(BuildContext context) {
    final theme = useProvider(themeProvider).theme;
    final showTapEffect = useState(false);

    final usedElevation = elevation ?? theme.card.elevation;
    final usedBorderRadius = borderRadius ?? theme.card.radius;

    return GestureDetector(
      onTap: onTap,
      onTapDown: (detail) {
        if (onTap != null) {
          showTapEffect.value = true;
        }
      },
      onTapUp: (detail) {
        if (onTap != null) {
          showTapEffect.value = false;
        }
      },
      onTapCancel: () {
        if (onTap != null) {
          showTapEffect.value = false;
        }
      },
      child: AnimatedOpacity(
        opacity: showTapEffect.value ? .50 : 1,
        duration: const Duration(milliseconds: 50),
        child: Card(
          margin: margin ?? EdgeInsets.zero,
          elevation: usedElevation,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(usedBorderRadius),
            side: borderWidth != null
                ? BorderSide(
                    color: theme.borderColor,
                    width: borderWidth!,
                  )
                : BorderSide.none,
          ),
          color: color,
          child: Container(
            padding: padding,
            child: child,
          ),
        ),
      ),
    );
  }
}
