import 'package:basic_ui/src/models/basic_theme.dart';
import 'package:basic_ui/src/providers/theme_provider.dart';
import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

import 'basic_appbar.dart';

/// Creates a visual scaffold for page.
class BasicScaffold extends HookWidget {
  const BasicScaffold({
    Key? key,
    this.body,
    this.drawer,
    this.bottomNavigationBar,
    this.title,
    this.actions,
    this.floatingActionButton,
    this.backgroundColor,
    this.enableSearchBar,
    this.searchBarPlaceHolder,
    this.onClear,
    this.onSubmitted,
    this.enableAppBar = true,
    this.tabs,
    this.tabBarViews,
    this.appBarTheme,
  }) : super(key: key);

  /// The primary content of the scaffold.
  final Widget? body;

  /// A panel displayed to the side of the [body], often hidden on mobile devices.
  /// Swipes in from either left-to-right ([TextDirection.ltr])
  /// or right-to-left ([TextDirection.rtl]).
  final Widget? drawer;

  /// A bottom navigation bar to display at the bottom of the scaffold.
  final Widget? bottomNavigationBar;

  /// Display title to the app bar.
  final String? title;

  /// A list of Widgets to display in a row after the [title].
  final List<Widget>? actions;

  /// A button displayed floating above [body], in the bottom right corner.
  final Widget? floatingActionButton;

  /// [body] background color.
  final Color? backgroundColor;

  /// Set this to true to make app bar title display search bar.
  final bool? enableSearchBar;

  /// Search bar placeholder
  final String? searchBarPlaceHolder;

  /// The callback that is called when the clear button is tapped.
  final Function? onClear;

  /// The callback when search query submitted.
  final Function(String)? onSubmitted;

  /// Set this to true to enable app bar.
  final bool enableAppBar;

  /// App bar theme.
  final BasicAppBarTheme? appBarTheme;

  /// The Widgets to display in tab bar,
  final List<Widget>? tabs;

  /// Creates a page view with one child per tab.
  ///
  /// The length of [tabBarViews] must be the same as the [tabs] length.
  final List<Widget>? tabBarViews;

  @override
  Widget build(BuildContext context) {
    final theme = useProvider(themeProvider).theme;

    final usedBackgroundColor = backgroundColor ?? theme.backgroundColor;

    if (tabs == null) {
      return _BuildScaffold(
        backgroundColor: usedBackgroundColor,
        theme: theme,
        enableAppBar: enableAppBar,
        title: title,
        actions: actions,
        enableSearchBar: enableSearchBar,
        searchBarPlaceHolder: searchBarPlaceHolder,
        onClear: onClear,
        onSubmitted: onSubmitted,
        drawer: drawer,
        body: body,
        floatingActionButton: floatingActionButton,
        bottomNavigationBar: bottomNavigationBar,
        appBarTheme: appBarTheme,
      );
    }

    return DefaultTabController(
      length: tabs!.length,
      child: Scaffold(
        backgroundColor: usedBackgroundColor,
        appBar: BasicAppBar(
          title: title ?? "",
          actions: actions,
          enableSearchBar: enableSearchBar ?? false,
          searchBarPlaceHolder: searchBarPlaceHolder,
          onClear: onClear,
          onSubmitted: onSubmitted,
          tabs: tabs,
          appBarTheme: appBarTheme,
        ),
        drawer: drawer,
        body: TabBarView(children: tabBarViews!),
        floatingActionButton: floatingActionButton,
      ),
    );
  }
}

class _BuildScaffold extends StatelessWidget {
  const _BuildScaffold({
    Key? key,
    this.backgroundColor,
    required this.theme,
    this.enableAppBar = false,
    this.title,
    this.actions,
    this.enableSearchBar,
    this.searchBarPlaceHolder,
    this.onClear,
    this.onSubmitted,
    this.appBarTheme,
    this.drawer,
    this.body,
    this.floatingActionButton,
    this.bottomNavigationBar,
  }) : super(key: key);

  final Color? backgroundColor;
  final BasicTheme theme;
  final bool enableAppBar;
  final String? title;
  final List<Widget>? actions;
  final bool? enableSearchBar;
  final String? searchBarPlaceHolder;
  final Function? onClear;
  final Function(String)? onSubmitted;
  final BasicAppBarTheme? appBarTheme;
  final Widget? drawer;
  final Widget? body;
  final Widget? floatingActionButton;
  final Widget? bottomNavigationBar;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: backgroundColor,
      appBar: enableAppBar
          ? BasicAppBar(
              title: title ?? "",
              actions: actions,
              enableSearchBar: enableSearchBar ?? false,
              searchBarPlaceHolder: searchBarPlaceHolder,
              onClear: onClear,
              onSubmitted: onSubmitted,
              appBarTheme: appBarTheme,
            )
          : null,
      drawer: drawer,
      body: body,
      floatingActionButton: floatingActionButton,
      bottomNavigationBar: bottomNavigationBar,
    );
  }
}
