import 'package:basic_ui/src/providers/theme_provider.dart';
import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

enum BasicTextVariant {
  tiny,
  small,
  medium,
  large,
  xLarge,
  xxLarge,
  mega,
  label
}

/// BasicText is a Widget for displaying text. You can use Text to standardize text across your app.
class BasicText extends HookWidget {
  const BasicText(
    this.text, {
    Key? key,
    this.variant = BasicTextVariant.medium,
    this.textAlign,
    this.color,
    this.fontWeight,
    this.fontSize,
    this.textOverflow,
    this.maxLines,
    this.softWrap,
  }) : super(key: key);

  /// A text to display.
  final String text;

  /// Text variant.
  final BasicTextVariant variant;

  /// How the text should be aligned horizontally.
  final TextAlign? textAlign;

  /// The color to use when painting the text.
  final Color? color;

  /// The thickness of the glyphs used to draw the text.
  final FontWeight? fontWeight;

  /// The size of glyphs (in logical pixels) to use when painting the text.
  final double? fontSize;

  /// How visual overflow should be handled.
  /// Defaults to retrieving the value from the nearest [DefaultTextStyle] ancestor
  final TextOverflow? textOverflow;

  /// An optional maximum number of lines for the text to span, wrapping if necessary. If the text exceeds the given number of lines, it will be truncated according to [overflow].
  ///
  /// If this is 1, text will not wrap. Otherwise, text will be wrapped at the edge of the box.
  final int? maxLines;

  ///   Whether the text should break at soft line breaks.
  ///
  /// If false, the glyphs in the text will be positioned as if there was unlimited horizontal space.
  final bool? softWrap;

  double getFontSize() {
    switch (variant) {
      case BasicTextVariant.label:
      case BasicTextVariant.tiny:
        return 13.0;

      case BasicTextVariant.small:
        return 14.0;

      case BasicTextVariant.medium:
        return 16.0;

      case BasicTextVariant.large:
        return 20.0;

      case BasicTextVariant.xLarge:
        return 22.0;

      case BasicTextVariant.xxLarge:
        return 30.0;

      case BasicTextVariant.mega:
        return 70.0;

      default:
        return 16.0;
    }
  }

  FontWeight getFontWeight() {
    switch (variant) {
      case BasicTextVariant.label:
        return FontWeight.w700;

      case BasicTextVariant.xLarge:
        return FontWeight.w600;

      case BasicTextVariant.xxLarge:
        return FontWeight.w600;

      case BasicTextVariant.mega:
        return FontWeight.w600;

      default:
        return FontWeight.w400;
    }
  }

  @override
  Widget build(BuildContext context) {
    final theme = useProvider(themeProvider).theme;

    return Text(
      text,
      style: TextStyle(
        fontWeight: fontWeight ?? getFontWeight(),
        fontSize: fontSize ?? getFontSize(),
        color: color ?? theme.fontColor,
      ),
      overflow: textOverflow,
      textAlign: textAlign,
      maxLines: maxLines,
      softWrap: softWrap,
    );
  }
}
