import 'package:basic_ui/basic_ui.dart';
import 'package:basic_ui/src/providers/theme_provider.dart';
import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

import 'basic_text.dart';

/// Creates a [FormField] that contains a [TextField].
class BasicTextField extends HookWidget {
  const BasicTextField({
    Key? key,
    this.label,
    this.placeholder,
    this.defaultValue,
    this.maxLength,
    this.maxLines = 1,
    this.autoClear,
    this.keyboardType,
    this.onChanged,
    this.onClear,
    this.validator,
    this.isPassword = false,
    this.fillColor,
    this.textColor,
    this.labelColor,
    this.borderColor,
    this.focusedBorderColor,
    this.borderWidth,
    this.borderRadius,
    this.contentPadding,
    this.labelVariant,
    this.textSize,
    this.margin,
    this.clearText = false,
    this.prefixIcon,
    this.hideCounterText = true,
    this.textAlign = TextAlign.start,
  }) : super(key: key);

  /// Text to display as [BasicTextField] label.
  final String? label;

  /// Text to display as [BasicTextField] hint text.
  final String? placeholder;

  /// Default value for [BasicTextField]
  final String? defaultValue;

  /// Set max length of [BasicTextField] value
  final int? maxLength;

  /// Set max lines of [BasicTextField]
  final int? maxLines;

  /// if not null and not 1, [BasicTextField] value will cleared.
  final int? autoClear;

  /// The type of information for which to optimize the text input control.
  ///
  /// On Android, behavior may vary across device and keyboard provider.
  ///
  /// This class stays as close to Enum interface as possible, and allows for additional
  /// flags for some input types. For example, numeric input can specify whether it supports
  /// decimal numbers and/or signed numbers.
  final TextInputType? keyboardType;

  /// The callback when [BasicTextField] value is changed.
  final Function(String)? onChanged;

  /// The callback when clear button is tapped.
  final Function()? onClear;

  /// The callback when [FormField] is validated.
  final String? Function(String?)? validator;

  /// Set this to true to make [BasicTextField] hide it's value
  final bool isPassword;

  /// The base fill color of the decoration's container color.
  final Color? fillColor;

  /// Text color.
  final Color? textColor;

  /// Label color.
  final Color? labelColor;

  /// Border color.
  final Color? borderColor;

  /// Border color when [BasicTextField] is focused.
  final Color? focusedBorderColor;

  /// Border width.
  final double? borderWidth;

  /// Border radius.
  final double? borderRadius;

  /// The padding for the input decoration's container.
  final EdgeInsetsGeometry? contentPadding;

  /// The [BasicTextVariant] for label text.
  final BasicTextVariant? labelVariant;

  /// Text size.
  final double? textSize;

  /// Empty space to surround the [BasicTextField].
  final EdgeInsets? margin;

  /// Set to true to enable clear text button.
  final bool clearText;

  /// Icon to show at start of [BasicTextField].
  final Icon? prefixIcon;

  /// Set to true to hide counter text.
  final bool hideCounterText;

  /// How the text should be aligned horizontally.
  final TextAlign textAlign;

  @override
  Widget build(BuildContext context) {
    final theme = useProvider(themeProvider).theme;
    final _isPasswordShow = useState(false);

    final controller = useState(TextEditingController());

    final usedFillColor = fillColor ?? theme.textField.fillColor;
    final usedTextColor = textColor ?? theme.textField.textColor;
    final usedLabelColor = labelColor ?? theme.textField.labelColor;
    final usedBorderColor = borderColor ?? theme.textField.borderColor;
    final usedFocusedBorderColor =
        focusedBorderColor ?? theme.textField.focusedBorderColor;
    final usedBorderWidth = borderWidth ?? theme.textField.borderWidth;
    final usedBorderRadius = borderRadius ?? theme.textField.borderRadius;
    final usedContentPadding = contentPadding ?? theme.textField.contentPadding;
    final usedLabelVariant = labelVariant ?? theme.textField.labelVariant;
    final usedTextSize = textSize ?? theme.textField.textSize;

    void onChanged(v) {
      controller.value.value.copyWith(text: v);
      if (this.onChanged != null) {
        this.onChanged!(v);
      }
    }

    void onClear() {
      controller.value.clear();
      if (this.onClear != null) {
        this.onClear!();
      }
    }

    useEffect(() {
      if (autoClear != null && autoClear != 1) {
        controller.value.clear();
      }
    }, [autoClear]);

    useEffect(() {
      controller.value.text = defaultValue ?? "";
    }, []);

    return Container(
      margin: margin,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          if (label != null)
            Padding(
              padding: const EdgeInsets.symmetric(vertical: 5),
              child: BasicText(
                label!,
                variant: usedLabelVariant,
                color: usedLabelColor,
              ),
            ),
          TextFormField(
            controller: controller.value,
            onChanged: onChanged,
            enableSuggestions: false,
            autocorrect: false,
            maxLength: maxLength,
            maxLines: maxLines,
            keyboardType: keyboardType,
            validator: validator,
            textAlign: textAlign,
            obscureText: isPassword && !_isPasswordShow.value,
            decoration: InputDecoration(
              counterText: hideCounterText ? "" : null,
              isDense: true,
              filled: usedFillColor != null,
              fillColor: usedFillColor,
              contentPadding: usedContentPadding,
              hintText: placeholder,
              focusedBorder: OutlineInputBorder(
                borderSide: BorderSide(
                  color: usedBorderWidth != 0
                      ? usedFocusedBorderColor ?? theme.primaryColor
                      : Colors.transparent,
                  width: usedBorderWidth != 0 ? usedBorderWidth + .5 : 0,
                ),
                borderRadius: BorderRadius.all(
                  Radius.circular(usedBorderRadius ?? 3),
                ),
              ),
              enabledBorder: OutlineInputBorder(
                borderSide: BorderSide(
                  color: usedBorderWidth != 0
                      ? usedBorderColor ?? theme.fontColor
                      : Colors.transparent,
                  width: usedBorderWidth,
                ),
                borderRadius: BorderRadius.all(
                  Radius.circular(usedBorderRadius ?? 3),
                ),
              ),
              errorBorder: OutlineInputBorder(
                borderSide: BorderSide(
                  color: usedBorderWidth != 0
                      ? usedBorderColor ?? theme.dangerColor
                      : Colors.transparent,
                  width: usedBorderWidth,
                ),
                borderRadius: BorderRadius.all(
                  Radius.circular(usedBorderRadius ?? 3),
                ),
              ),
              focusedErrorBorder: OutlineInputBorder(
                borderSide: BorderSide(
                  color: usedBorderWidth != 0
                      ? usedBorderColor ?? theme.dangerColor
                      : Colors.transparent,
                  width: usedBorderWidth,
                ),
                borderRadius: BorderRadius.all(
                  Radius.circular(usedBorderRadius ?? 3),
                ),
              ),
              errorStyle: TextStyle(color: theme.dangerColor),
              suffixIcon: isPassword
                  ? BasicTap(
                      onTap: () {
                        _isPasswordShow.value = !_isPasswordShow.value;
                      },
                      child: Icon(
                        _isPasswordShow.value
                            ? Icons.visibility_off
                            : Icons.visibility,
                        color: usedTextColor ?? theme.fontColor,
                      ),
                    )
                  : clearText
                      ? BasicTap(
                          onTap: onClear,
                          child: Icon(
                            Icons.close,
                            color: usedTextColor ?? theme.fontColor,
                          ),
                        )
                      : null,
              suffixIconConstraints: BoxConstraints(
                minWidth: usedContentPadding.horizontal + 29,
                minHeight: usedContentPadding.vertical + 29,
              ),
              prefixIcon: prefixIcon != null
                  ? Padding(
                      padding: const EdgeInsetsDirectional.only(end: 16.0),
                      child: prefixIcon,
                    )
                  : null,
              prefixIconConstraints: const BoxConstraints(),
            ),
            style: TextStyle(
              fontSize: usedTextSize,
              color: usedTextColor ?? theme.fontColor,
            ),
          ),
        ],
      ),
    );
  }
}
